#include <stdio.h>
#include <assert.h>

#include "err.h"
#include "thingstack.h"
#include "lrsserv.h"

static  lrs_dic *P;	/* structure for holding current dictionary and indices  */
static  lrs_dat *Q;	/* structure for holding static problem data             */

static lrs_mp_matrix Lin;

FILE *debugfp;

static int num_orbits=0;
static int *orbit=NULL;
static int *orbit_start=NULL;


/* Internal functions */

static void retcode(int);
static void putline(char *);
static void putstring(char *);
static void put_mp_vec(lrs_mp_vector, int);
static void put_long_vec(long *, int);
static void putmp(lrs_mp);
static void putint(int);
static void endline();
static long find_cob(int);
static long find_bas(int);

static void lorbitratio(int i, int keep_dvars,
			long* count_p, long *start_p, long *out);
static void lpivot(long bas, long cob);

static int *lex_signs=NULL;

static void logmp(lrs_mp num){
  char buf[BUFSIZ];
  if (!negative(num))
    logprintf(" ",buf);    
  /* hmm, no buffer overflow checking */

  mptoa(num,buf);
  logprintf("%s",buf);
  
}

/* callbacks */
extern void shutdown(){
  putline("BYE");
  exit(0);
}

/** 
 * m n init
 * 
 */
extern void init(){
  int m,n;

  dlog("K=>init");
  n=pop_int();
  m=pop_int();
  
  Q->m=m;
  Q->n=n;

  P = lrs_alloc_dic(Q);
  retcode(P!=NULL);
}

/** 
 * lponly
 * 
 */
extern void lponly(){
  Q->lponly=1;
}



/** 
 * debug
 * 
 */
extern void debug(){
  Q->debug=1;
}

extern void geometric(){
  Q->geometric=1;
}


extern void set_orbits(){
  int m=Q->m;

  dlog("K=>set_orbits");
  orbit=calloc(m+1,sizeof(int));
  orbit_start=calloc(m+1,sizeof(int));

  num_orbits=0;
  while (!top_is_mark()){
    int j=orbit_start[num_orbits];
    while(!top_is_mark()){
      orbit[j++]=pop_int();
    }
    pop();
    num_orbits++;
    orbit_start[num_orbits]=j;
  }
  pop();
}


extern void dump_orbits(){
  int i,j;
  
  dlog("K=>dump_orbits");
  for (i=0; i< num_orbits; i++){
    dlog("orbit %d = ",i);
    for (j=orbit_start[i]; j< orbit_start[i+1]; j++){
      logprintf("%d ",orbit[j]);
    }
    logprintf("\n");
  }

}


/* @ v1(int) ... vk(int) lex_signs */
extern void set_lex_signs(){
  int  num_signs;
  dlog("K=>set_lex_signs");

  lex_signs=calloc(P->m+1,sizeof(int));

  num_signs=0;

  while(!top_is_mark()){
    lex_signs[num_signs++]=pop_int();
  }
  pop();
}

extern void dump_lex_signs(){
  int i;

  for (i=0; i< Q->m; i++)
    printf("%d ",lex_signs[i]);

  printf("\n");
}

/** 
 * coeffs:(rational x n)  row(int) type(1=GE, 0=EQ) set_row
 * 
 */

extern void set_row(){
  int type, row;

  lrs_mp_vector num=lrs_alloc_mp_vector(Q->n);
  lrs_mp_vector den=lrs_alloc_mp_vector(Q->n);

  dlog("K=>set_row");
  type=pop_int();
  row=pop_int();

  pop_rational_vector(num,den,Q->n);
  /*   fprintf(stderr,"\nrow=");
  for (j=0; j<Q->n; j++)
    prat("",num[j],den[j]);
  fprintf(stderr,"\n");
  */
  lrs_set_row_mp(P,Q,row,num,den,type);
  lrs_clear_mp_vector(num,Q->n);
  lrs_clear_mp_vector(den,Q->n);
}

/** 
 * coeffs:(rational x n)  type(1=maximize, 0=minimize) set_obj
 * 
 */

extern void set_obj(){
  int type,j;
  lrs_mp_vector num=lrs_alloc_mp_vector(Q->n);
  lrs_mp_vector den=lrs_alloc_mp_vector(Q->n);

  dlog("K=>set_obj");
  type=pop_int();

  pop_rational_vector(num,den,Q->n);

  if (type==MAXIMIZE) {
    Q->maximize=TRUE;
  } else {
    Q->minimize=TRUE;
    for (j=0; j< Q->n; j++){
      storesign(num[j],NEG);
    }
  }

  lrs_set_row_mp(P,Q,0,num,den,type);
  lrs_clear_mp_vector(num,Q->n);
  lrs_clear_mp_vector(den,Q->n);

}

/** 
 * () solve_lp
 * 
 */
extern void solve_lp(){
  dlog("K=>solve_lp");
  retcode(lrs_solve_lp(P,Q));
}
/** 
 * col:int getsolution
 * 
 */
extern void getsolution(){
  int col,ret;
  lrs_mp_vector output=lrs_alloc_mp_vector(Q->n);

  dlog("K=>getsolution");
  col=pop_int();
  if (col>P->d || col < 0)
    err_fatal("getsolution","illegal column %d\n",col);

  ret=lrs_getsolution(P,Q,output,col);

  if (ret){
    putline("OK");
    put_mp_vec(output,Q->n);
  } else {
    putline("NO");
  }
  lrs_clear_mp_vector(output,Q->n);
}

/*
 * getvertex: don't try to be clever about lex minimality or hull
 */
extern void getvertex(){
  int i,ired,ind;
  long *redundcol = Q->redundcol;
  lrs_mp_vector output=lrs_alloc_mp_vector(Q->n);


  /* copy column 0 to output */


  i = 1;
  ired = 0;
  copy (output[0], P->det);


  for (ind = 1; ind < Q->n; ind++)	/* extract solution */

    if ((ired < Q->nredundcol) && (redundcol[ired] == ind))
      /* column was deleted as redundant */
      {
	itomp (ZERO, output[ind]);
	ired++;
      }
    else
      /* column not deleted as redundant */
      {
	getnextoutput (P, Q, i, ZERO, output[ind]);
	i++;
      }




  reducearray (output, Q->n);

  put_mp_vec(output,Q->n);
  lrs_clear_mp_vector(output,Q->n);


}

extern void getfirstbasis(){
  int rval;
  dlog("K=>getfirstbasis");
  rval=lrs_getfirstbasis(&P,Q,&Lin,TRUE);
  if (rval){
    putint(P->d);
    endline();
  }

}

extern void getlinearities(){
  int startcol=0,nlin,col;
  if (Q->homogeneous && Q->hull)
    startcol++;			/* col zero not treated as redundant   */

  nlin=Q->nredundcol - startcol;
  if (nlin>0){
    putline("OK");
    putint(nlin);
    endline();
    for (col = startcol; col < Q->nredundcol; col++)	
      put_mp_vec(Lin[col],Q->n);	

  } else {
    putline("NO");
  }

}

/** 
 * () h_rep
 * 
 */

extern void v_rep(){
  assert(Q);
  Q->hull=TRUE;
  Q->polytope=TRUE;
}

/** 
 * () h_rep
 * 
 */
extern void h_rep(){
  assert(Q);
  Q->hull=FALSE;
}

extern void getnextbasis(){
  retcode(lrs_getnextbasis(&P,Q,FALSE));
}

/* a scrap of lrs_getfirstbasis, for debugging */

extern void initinequality(){
  long *inequality = Q->inequality;
  long *linearity = Q->linearity;
  long nlinearity=Q->nlinearity;
  int m=Q->m;
  int d=P->d;
  int i,j,k;
  
  for (i = 0; i < nlinearity; i++)	/* put linearities first in the order */
    inequality[i] = linearity[i];

  k = 0;			/* index for linearity array   */

  if (Q->givenstart)
    k = d;
  else
    k = nlinearity;
  for (i = m; i >= 1; i--)
    {
      j = 0;
      while (j < k && inequality[j] != i)
	j++;			/* see if i is in inequality  */
      if (j == k)
	inequality[k++] = i;
    }
  if (Q->debug)
    {
      fprintf (lrs_ofp, "\n*Starting cobasis uses input row order");
      for (i = 0; i < m; i++)
	fprintf (lrs_ofp, " %ld", inequality[i]);
    }
}

extern void renumberinequality(){
  long *inequality = Q->inequality;
  long *linearity = Q->linearity;
  long nlinearity=Q->nlinearity;
  int m=Q->m;
  int d=P->d;
  int lastdv=Q->lastdv;
  int nredundcol=Q->nredundcol;
  long *B = P->B;
  long *C = P->C;
  long *Col = P->Col;
  int i,j,k;


  for (i = 1; i <= m; i++)
    inequality[i] = i;
  if (nlinearity > 0)		/* some cobasic indices will be removed */
    {
      for (i = 0; i < nlinearity; i++)	/* remove input linearity indices */
	inequality[linearity[i]] = 0;
      k = 1;			/* counter for linearities         */
      for (i = 1; i <= m - nlinearity; i++)
	{
	  while (k <= m && inequality[k] == 0)
	    k++;		/* skip zeroes in corr. to linearity */
	  inequality[i] = inequality[k++];
	}
    }				/* end if linearity */
  if (Q->debug)
    {
      fprintf (lrs_ofp, "\ninequality array initialization:");
      for (i = 1; i <= m - nlinearity; i++)
	fprintf (lrs_ofp, " %ld", inequality[i]);
    }
  if (nredundcol > 0)
    {
      Lin = lrs_alloc_mp_matrix (nredundcol, Q->n);

      for (i = 0; i < nredundcol; i++)
	{
	  if (!(Q->homogeneous && Q->hull && i == 0))	/* skip redund col 1 for homog. hull */
	    {
	      lrs_getray (P, Q, Col[0], P->C[0] + i - Q->hull, Lin[i]);		/* adjust index for deletions */
	    }

	  if (!removecobasicindex (P, Q, 0L))
	    err_fatal("renumberinequality","Failed to remove cobasic index");
 
	}
    }				/* end if nredundcol > 0 */

  while (C[0] <= m)
    {
      i = C[0];
      j = inequality[B[i] - lastdv];
      inequality[B[i] - lastdv] = inequality[C[0] - lastdv];
      inequality[C[0] - lastdv] = j;
      C[0] = B[i];
      B[i] = i;
      reorder1 (C, Col, ZERO, d);
    }

  if (Q->debug)
    {
      fprintf (lrs_ofp, "\n*Inequality numbers for indices %d .. %d : ", lastdv + 1, m + d);
      for (i = 1; i <= m - nlinearity; i++)
	fprintf (lrs_ofp, " %ld ", inequality[i]);
      printA (P, Q);
    }
}

/** 
 * 
 * col getcobasis
 * 
 */

void 
getcobasis ()
/* col is output column being printed */
{
  long i;
  long rflag;			/* used to find inequality number for ray column */
/* assign local variables to structures */
  lrs_mp_matrix A = P->A;
  long *B = P->B;
  long *C = P->C;
  long *Col = P->Col;
  long *Row = P->Row;
  long *inequality = Q->inequality;
  long *temparray = Q->temparray;
  long d = P->d;
  long lastdv = Q->lastdv;
  long m=P->m;
  long nincidence;       /* count number of tight inequalities */
  int  col,inc_count;

  long *extra_incidences=malloc(m*sizeof(long));


  col=pop_int();

  dlog("K=>getcobasis");
  rflag = (-1);
  for (i = 0; i < d; i++){
    if (Col[i] == col)
      rflag = temparray[i];	/* look for ray index */

    temparray[i] = inequality[C[i] - lastdv];
  }

/*   for (i = 0; i < d; i++) */
/*     reorder (temparray, d); */


  if ( col == 0 )
     nincidence = d;
  else
     nincidence = d-1;

  inc_count=0;
  for(i=lastdv+1;i<=m;i++){
    if ( zero (A[Row[i]][0] )){
      if( ( col == ZERO ) || zero (A[Row[i]] [col]) ){
	extra_incidences[inc_count]=inequality[B[i] - lastdv ];
	nincidence++;
	inc_count++;
      }
    }
  }

  putmp(P->det); endline();
  putint(rflag); endline ();
  put_long_vec(temparray,d);
  putint(nincidence); endline();
  put_long_vec(extra_incidences,inc_count);
  free(extra_incidences);

}				


void 
dump_dict(){
  printA(P,Q);
}

void 
get_dict(){
    long i, j;
/* assign local variables to structures */
  lrs_mp_matrix A = P->A;
  long *B = P->B;
  long *C = P->C;
  long *Row = P->Row;
  long *Col = P->Col;
  long m, d, lastdv;
  m = P->m;
  d = P->d;
  lastdv = Q->lastdv;

  putmp (P->det); endline();
  
  put_long_vec(B,m+1);
  put_long_vec(Row,m+1);
  put_long_vec(C,d+1);
  put_long_vec(Col,d+1);


  i=0;
  while ( i<= m )
    {
      putint(d+1);
      for (j = 0; j <= d; j++)
	putmp (A[Row[i]][Col[j]]);
      endline();

      if (i==0 && Q->nonnegative)  /* skip basic rows - don't exist! */
          i=d;
      i++;
    }
  putline("DONE");
}

void 
raw_dict(){
    long i, j;
/* assign local variables to structures */
  lrs_mp_matrix A = P->A;
  long *B = P->B;
  long *C = P->C;
  long *Row = P->Row;
  long *Col = P->Col;
  long m, d, lastdv;
  m = P->m;
  d = P->d;
  lastdv = Q->lastdv;

  putmp (P->det); endline();
  
  put_long_vec(B,m+1);
  put_long_vec(Row,m+1);
  put_long_vec(C,d+1);
  put_long_vec(Col,d+1);


  i=0;
  while ( i<= m )
    {
      putint(d+1);
      for (j = 0; j <= d; j++)
	putmp (A[i][j]);
      endline();

      if (i==0 && Q->nonnegative)  /* skip basic rows - don't exist! */
          i=d;
      i++;
    }
  putline("DONE");
}


void
lexratio ()	/*find lex min. ratio */
                /* find min index ratio -aig/ais, ais<0 */
{
  int facet,col,cob,enter;
  long *Col = P->Col;
  long *B = P->B;
  long *inequality = Q->inequality;
  long lastdv=Q->lastdv;


  if (Q->debug)
    printA(P,Q);

  facet=pop_int();
  dlog("K=>lexratio");
  if ((cob=find_cob(facet)) < 0){
    retcode(0);
    return;
  }

  col = Col[cob];
  /*  dlog("# lexratio2 facet=%d cobindex=%d col=%d",facet,cob,col); */

  enter = ratio(P,Q,col);
  if (enter>0) {
    putline("OK");
    putint(inequality[B[enter]-lastdv]);
    endline();
  } else {
    putline("NO");
  }
}


/* modified from lrslib to skip lex ratio testing */

void
allratio ()
{
  long i, j, comp, ratiocol, basicindex, start, nstart, cindex, bindex;
  long firstime;		/*For ratio test, true on first pass,else false */
  lrs_mp Nmin, Dmin;
  long degencount, ndegencount;
/* assign local variables to structures */
  lrs_mp_matrix A = P->A;
  long *B = P->B;
  long *Row = P->Row;
  long *Col = P->Col;
  long *minratio = Q->minratio;
  long *inequality = Q->inequality;
  long m, d, lastdv;
  long col,cob,facet;



  if (Q->debug)
    printA(P,Q);

  facet=pop_int();
  dlog("K=>allratio");
  if ((cob=find_cob(facet)) < 0){
    dlog("# find_cob failed");
    retcode(0);
    return;
  }

  col = Col[cob];


  if (Q->debug)
    dlog("allratio2 facet=%d cobindex=%d col=%d",facet,cob,col); 

  m = P->m;
  d = P->d;
  lastdv = Q->lastdv;

  nstart=0;
  ndegencount=0;
  degencount = 0;
  for (j = lastdv + 1; j <= m; j++)    {
      /* search rows with negative coefficient in dictionary */
      /*  minratio contains indices of min ratio cols        */
      if (negative (A[Row[j]][col]))
	minratio[degencount++] = j;
  }				/* end of for loop */
  if (Q->debug)    {
      fprintf (lrs_ofp, "  potential min ratios: ");
      for (i = 0; i < degencount; i++)
	fprintf (lrs_ofp, " %ld ", B[minratio[i]]);
      printA(P,Q);
  }

  lrs_alloc_mp(Nmin); lrs_alloc_mp(Dmin);
  ratiocol = 0;			/* column being checked, initially rhs */
  start = 0;			/* starting location in minratio array */
  bindex = d + 1;		/* index of next basic variable to consider */
  cindex = 0;			/* index of next cobasic variable to consider */
  basicindex = d;		/* index of basis inverse for current ratio test, except d=rhs test */
  
  if (B[bindex] == basicindex)	/* identity col in basis inverse */
    {
      if (minratio[start] == bindex)
	/* remove this index, all others stay */
	{
	  start++;
	  degencount--;
	}
      bindex++;
    }
  else
    /* perform ratio test on rhs or column of basis inverse */
    {

      firstime = TRUE;
      /*get next ratio column and increment cindex */
      for (j = start; j < start + degencount; j++){
	i = Row[minratio[j]];	/* i is the row location of the next basic variable */
	comp = 1;		/* 1:  lhs>rhs;  0:lhs=rhs; -1: lhs<rhs */
	if (firstime)
	  firstime = FALSE;	/*force new min ratio on first time */
	else  {
	  if (positive (Nmin) || negative (A[i][ratiocol]))	  {
	    if (negative (Nmin) || positive (A[i][ratiocol]))
	      comp = comprod (Nmin, A[i][col], A[i][ratiocol], Dmin);
	    else
	      comp = -1;
	  }  else if (zero (Nmin) && zero (A[i][ratiocol]))
	    comp = 0;
	  
	  if (ratiocol == ZERO)
	    comp = -comp;	/* all signs reversed for rhs */
	}
	if (comp == 1)	{		/*new minimum ratio */
	  nstart = j;
	  copy (Nmin, A[i][ratiocol]);
	  copy (Dmin, A[i][col]);
	  ndegencount = 1;
	} else {
	  if (comp == 0){	/* repeated minimum */
	    minratio[nstart + ndegencount++] = minratio[j];
	  }
	}
      }
    }

  degencount = ndegencount;
  start = nstart;

  if (Q->debug){
    fprintf (lrs_ofp, " allratio: final degencount=%ld ", degencount);
    fprintf (lrs_ofp, "  Min ratios: ");
    for (i = start; i < start + degencount; i++)
      fprintf (lrs_ofp, " B[%ld]=%ld (%ld, %ld)", minratio[i],B[minratio[i]], inequality[B[minratio[i] -lastdv ]], lastdv);
  }
				/*end of while loop */
  lrs_clear_mp(Nmin); lrs_clear_mp(Dmin);
  
  putint(degencount);
  for (i = start; i < start + degencount; i++){
    putint(inequality[B[minratio[i]]-lastdv]);
  }
  endline();
}


static int get_lex_sign(int row){
  if (lex_signs==NULL) 
    return 1;

  return lex_signs[row];
}
static void orbit_sum(lrs_mp sum, int basisrow, int orb, int *col_loc){
  int m=Q->m;

  long *Col = P->Col;
  long *Row = P->Row;
  lrs_mp_matrix A= P->A;

  itomp(0,sum);

  assert(col_loc);
  /* apply each orbit to each row of the basis inverse */
  int k;
  for (k=orbit_start[orb]; k< orbit_start[orb+1]; k++){
    /* find the next column in the orbit */
    int ocol,cindex;
    assert(k<=m);
    ocol=orbit[k];
    
    cindex=col_loc[ocol];
    assert(ocol<=m);
    if (cindex>=0){
      /* Non identity column, stored as 
       * column of -$A^{-1}_B A_N$
       */
      if (Q->debug){
	logprintf("adding A[%d][%d]=",basisrow,ocol);
	logmp(A[Row[basisrow]][Col[cindex]]);
	logprintf("\n");
      }
      linint(sum,1,A[Row[basisrow]][Col[cindex]],1);	  
    } else {
      /* identity column, treated implicitly */
      if (-1*cindex==basisrow){
	linint(sum,1,P->det,-1);	  
	if (Q->debug){
	  logprintf("adding I[%d,%d] =-1*",basisrow,ocol);
	  logmp(P->det);
	  logprintf("\n");
	}
      } else {
	if (Q->debug)
	  logprintf("skipping I[%d,%d]\n",basisrow,ocol);
      }
    }
  }
  storesign(sum,-1*get_lex_sign(orb));
}


  /* figure out how to reconstruct the inverse basis matrix 
   * note that this is a different basis inverse than in lex ratio testing, 
   * in particular it is the basis that gets us back to the original slack 
   * formulation.

   * It follows that the default basis will have negative entries in all 
   * the leading positions of all of the non-decision rows. This is 
   * a puzzle.
   */

extern int* find_inverse_cols(){
  long *inequality=Q->inequality;
  long *C = P->C;
  long *B = P->B;
  long  m = P->m;
  long d=P->d;

  /* this should really be some global or 
     kept in lrs structure 
  */

  int *col_loc=calloc(m+1,sizeof(int));
  int bindex;
  int cindex;
  int col;
  int lastbasicdv;

  if (Q->debug){
    int i;
    logprintf("# inequality= ");
    for (i=0; i<=m; i++)
      logprintf("%d ",inequality[i]);
    logprintf("\n");
  }
  lastbasicdv=0;
  while(lastbasicdv<d && B[lastbasicdv+1]==lastbasicdv+1)
    lastbasicdv++;

  dlog("last basic decision var %d\n",lastbasicdv);

  bindex=lastbasicdv+1;
  cindex=d-lastbasicdv;

  for (col=1; col <= m; col++){
    if (C[cindex]==inequality[col]+d){
      col_loc[col]=cindex++;
    } else {
      col_loc[col]=-1*(bindex);
      bindex++;
    }
  }
  if (Q->debug){
    int j;
    for (j=0; j<=m; j++){
      dlog("col_loc[%d]=%d",j,col_loc[j]);
    }
  }

  return col_loc;
}

/* 
   Do partial lexicographic pivoting, where the inequalities 
   in each orbit are symbollically perturbed the same amount.

   leave(int)  orbitratio
*/

extern
void orbitratio(){
  int facet=pop_int();
  int cob;
  long *minratio = Q->minratio;
  long *inequality = Q->inequality;
  long *B = P->B;
  long  lastdv = Q->lastdv;

  long degencount,start;

  dlog("K=>orbitratio");

  if ((cob=find_cob(facet)) < 0){
    dlog("# find_cob failed");
    retcode(0);
    return;
  }



  lorbitratio(cob,1,&degencount,&start,minratio);

  putint(degencount);
  {
    int i;
    for (i = start; i < start + degencount; i++){
      putint(inequality[B[minratio[i]]-lastdv]);
    }
  }
  endline();
}

extern
void iorbitratio(){
  long degencount, start;
  long *minratio = Q->minratio;



  int keep_dvars=pop_int();
  int cob=pop_int();


  lorbitratio(cob,keep_dvars,&degencount,&start,minratio);

  putint(degencount);
  {
    int i;
    for (i = start; i < start + degencount; i++){
      putint(minratio[i]);
    }
  }
  endline();

}


extern void
lorbitratio (int cob, int keep_dvars,long *count_p, 
	     long* start_p, long *minratio)
{
  long j,  comp, orbitindex, start, nstart;

  long firstime;		/*For ratio test, true on first pass,else false */
  lrs_mp Nmin, Dmin,colSum,mpOne;
  long degencount, ndegencount;
/* assign local variables to structures */
  lrs_mp_matrix A = P->A;
  long *B = P->B;

  long *Row = P->Row;
  long *Col = P->Col;
  long *inequality = Q->inequality;
  long col;
  long  m = P->m;
  long d=P->d;
  long  lastdv = Q->lastdv;

  int *col_loc;

  lrs_alloc_mp(mpOne);
  itomp(1,mpOne);

  if (Q->debug)
    printA(P,Q);

  assert (cob < d);

  col = Col[cob];

  assert(col <= d);

  if (Q->debug)
    dlog("orbitratio2 cobindex=%d col=%d",cob,col); 

  nstart=0;
  ndegencount=0;
  degencount = 0;
  
  for (j = ( keep_dvars ? lastdv + 1 : 1) ; j <= m; j++)    {
    /* search rows with negative coefficient in dictionary */
    /*  minratio contains indices of min ratio cols        */
    if (negative (A[Row[j]][col]))
      minratio[degencount++] = j;
  }				/* end of for loop */
  if (Q->debug)    {
    fprintf (lrs_ofp, "  potential min ratios: ");
    int i;
    for (i = 0; i < degencount; i++)
      fprintf (lrs_ofp, "B[%ld]= %ld ", 
	       minratio[i],
	       B[minratio[i]]);
    printA(P,Q);
  }

  lrs_alloc_mp(Nmin); lrs_alloc_mp(Dmin);
  lrs_alloc_mp(colSum); /* sum of columns with the same epsilon value */
  start = 0;			/* starting location in minratio array */

  orbitindex=-1;                 /* orbit being checked, -1=rhs */

  col_loc=find_inverse_cols();

  while (degencount > 1 && orbitindex < num_orbits) {
    /* perform ratio test on rhs or column(s) of B^-1 * Epsilon */

    firstime=TRUE;

    for (j = start; j < start + degencount; j++){
      /* i is the row location of the next basic variable */
      int i;
      i = Row[minratio[j]];	
      if (orbitindex < 0){
	copy(colSum,A[i][0]);
      } else {
	orbit_sum(colSum,minratio[j],orbitindex,col_loc);
      }
      if (Q->debug){
	logprintf("orbsum[%d,%d]=",minratio[j],orbitindex);
	logmp(colSum);
	logprintf("\n");
      }

      comp = 1;		/* 1:  lhs>rhs;  0:lhs=rhs; -1: lhs<rhs */
      if (firstime)
	firstime = FALSE;	/*force new min ratio on first time */
      else  {
	if (positive (Nmin) || negative (colSum))	  {
	  if (negative (Nmin) || positive (colSum))
	    comp = comprod (Nmin, A[i][col], colSum, Dmin);
	  else
	      comp = -1;
	  }  else if (zero (Nmin) && zero (colSum))
	    comp = 0;
	  
	  if (orbitindex<0 )
	    comp = -comp;	/* all signs reversed for rhs */
	}
	if (comp == 1)	{		/*new minimum ratio */
		 
	  nstart = j;
	  copy (Nmin, colSum);
	  copy (Dmin, A[i][col]);
	  ndegencount = 1;
	  if (Q->debug){
	    char* num=calloc(BUFSIZ,sizeof(char));
	    char* den=calloc(BUFSIZ,sizeof(char));

	    mptoa(Dmin,den);
	    mptoa(Nmin,num);

	    dlog("New minimum %s/%s",num,den);
	    free(num);
	    free(den);
	  }

	} else {
	  if (comp == 0){	/* repeated minimum */
	    minratio[nstart + ndegencount++] = minratio[j];
	  }
	}
	
      }
      degencount = ndegencount;
      start = nstart;

      if (Q->debug){
	int i;
	logprintf ( " orbit=%ld degencount=%ld\n", orbitindex, degencount);
	logprintf ( "  Min ratios: ");
	for (i = start; i < start + degencount; i++)
	  logprintf (" B[%ld]=%ld (%ld, %ld)", minratio[i],B[minratio[i]], inequality[B[minratio[i] -lastdv ]], lastdv);
	logprintf("\n");
      }
          
      if (Q->debug)
	logprintf("\n");

      orbitindex++;
      assert(orbitindex<m);
    }
  
    if (Q->debug){
      int i;
      dlog ( " final degencount=%ld ", degencount);
      dlog ("  Min ratios: ");
      for (i = start; i < start + degencount; i++)
	dlog ( " %ld (%ld, %ld)", B[minratio[i]], inequality[B[minratio[i] -lastdv ]], lastdv);
    }

    
			/*end of while loop */
  lrs_clear_mp(Nmin); lrs_clear_mp(Dmin); lrs_clear_mp(colSum);
  free(col_loc);
  
  *count_p=degencount;
  *start_p=start;
}


/*
 * getorbitbasis
 *
 * find a basis that is lex positive with respect to the given orbits 
 * and lex_signs
 * 
 * based on primalfeasible from lrslib.c
 * should be called after getfirstbasis, which is a bit wasteful.
 */

#if 0

extern void getorbitbasis(){

  long primalinfeasible = TRUE;
  long i, j;
/* assign local variables to structures */
  lrs_mp_matrix A = P->A;
  long *Row = P->Row;
  long *Col = P->Col;
  long m, d, lastdv;
  m = P->m;
  d = P->d;
  lastdv = Q->lastdv;

/* Do dual pivots to get primal feasibility */
/* Note that cost row is all zero, so no ratio test needed for Dual Bland's rule */

/*temporary: try to get new start after linearity */

  while (primalinfeasible)
    {
      i=lastdv+1;
      while (i <= m && orbit_lex_positive(A,i))
        i++;
      if (i <= m )
	{
	      j = 0;		/*find a positive entry for in row */
	      while (j < d && !positive (A[Row[i]][Col[j]]))
		j++;
	      if (j >= d)
		return (FALSE);	/* no positive entry */
	      pivot (P, Q, i, j);
	      update (P, Q, &i, &j);
        }
      else
         primalinfeasible = FALSE;
    }				/* end of while primalinfeasibile */
  return (TRUE);
}				/* end of primalfeasible */

#endif
/*
 *  leave enter pivot
 */
extern void dopivot(){
  int leave, enter;
  long cob,bas;

  enter=pop_int();
  leave=pop_int();

  dlog("K=>pivot");
  cob=find_cob(leave);
  if (cob<0)  {
    retcode(0);
    return;
  }
  bas=find_bas(enter);
  if (bas<0){
    retcode(0);
    return;
  }
  lpivot(bas,cob);
  retcode(1);
}

/* cobasic(int) basic(int)  ipivot
 *
 */

extern void ipivot(){
  long cob,bas;

  bas=pop_int();
  cob=pop_int();


  lpivot(bas,cob);
  retcode(1);
}




static void lpivot(long bas , long cob){
  pivot(P,Q,bas,cob);
  update(P,Q,&bas,&cob);
}

static long find_bas(int enter){
  int j;
  long lastdv=Q->lastdv;
  long *inequality=Q->inequality;
  long *B=P->B;
  long m=P->m;

  for (j=lastdv+1; j<= m && inequality[B[j]-lastdv] != enter; j++);

  if ((inequality[B[j]-lastdv])!= enter)
    return -1;
  return j;

  
  
}
static long find_cob(int leave){
  int j;
  long lastdv=Q->lastdv;
  long *inequality=Q->inequality;
  long *C=P->C;
  
  long d=P->d;

  for (j=0; j< d && inequality[C[j]-lastdv]!= leave; j++);

  if (inequality[C[j]-lastdv]!= leave)
    return -1;
  return j;
  
}
/***********************************************************************/
/* utility routines */

static void put_mp_vec(lrs_mp_vector v, int n){
  int j;
  putint(n);
  for (j=0; j<n; j++){
    putmp(v[j]);
  }
  endline();
}

static void put_long_vec(long *v, int n){
  int j;
  putint(n);
  for (j=0; j<n; j++){
    putint(v[j]);
  }
  endline();
}

static void putline(char *s){
  dlog("<=%s",s);
  puts(s);
  fflush(stdout);
}

static void endline(){
  dlog("<=\\n");
  fputs("\n",stdout);
  fflush(stdout);
}

static void putstring(char *s){
  dlog("<=%s",s);
  fputs(s,stdout);
  fputs(" ",stdout);
}

static void putint(int i){
  dlog("<=%d",i);
  printf("%d ",i);
}

static void putmp(lrs_mp a){
  char buf[BUFSIZ];
  mptoa(a,buf);
  putstring(buf);
}

static void retcode(int r){
  if (r){
    putline("OK");
  } else {
    putline("FAIL");
  }
}

/*
 * based on readfacets()
 */
int long_cmp(const void *a_p, const void *b_p){
  return *(long*)a_p-*(long*)b_p;
}
extern void setcobasis(){
/* read and check facet list for obvious errors during start/restart */
/* this must be done after linearity option is processed!! */
  long i, j;
/* assign local variables to structures */
  long m, d;
  long *linearity = Q->linearity;
  long *facet=Q->facet;
  m = Q->m;
  d = P->d;

  dlog("K=>setcobasis");
  for (j = Q->nlinearity; j < d; j++)	/* note we place these after the linearity indices */
    {
      facet[j]=pop_int();
      if (facet[j] < 1 || facet[j] > m)
	{
	  dlog ( "\n Start/Restart cobasic indices must be in range 1 .. %ld ", m);
	  retcode(0);
	  return;
	}
      for (i = 0; i < Q->nlinearity; i++)
	if (linearity[i] == facet[j])
	  {
	    dlog ("\n Start/Restart cobasic indices should not include linearities");
	    retcode(0);
	    return;
	  }
      for (i = 0; i < j; i++)
	if (facet[i] == facet[j])
	  {
	    dlog( "\n  Start/Restart cobasic indices must be distinct");
	    retcode(0);
	    return;
	  }
    }
  Q->restart=1;
  retcode(restartpivots (P, Q));
}



extern FILE *yyin;
int main(int argc, char **argv){


  if (argc>1){
    yyin=fopen(argv[1],"r");
  }

  if (argc>2){
    freopen(argv[2],"w",stdout);
  }

#ifdef DEBUGLOG
  {
    char buf[BUFSIZ];
    sprintf(buf,"/tmp/lrsserv%d.log",getpid());
    debugfp=freopen(buf,"w",stderr);
    lrs_ofp=debugfp;
    setvbuf(debugfp,NULL,_IOLBF,BUFSIZ);
  }
#endif
  
  setvbuf(stdout,NULL,_IOLBF,BUFSIZ);

  lrs_init_quiet(stdin,stderr);

  Q = lrs_alloc_dat ("LRS globals");
  assert(Q!=NULL);

  putline("HELLO");
  yylex();
  return 0;
}

